@@ -5,6 +5,7 @@ from importlib.util import find_spec
 from pathlib import Path
 
 import matplotlib
+import numpy
 import pytest
 import yaml
 from packaging.version import Version
@@ -20,6 +21,7 @@ from yt.utilities.answer_testing.testing_utilities import (
 )
 
 MPL_VERSION = Version(matplotlib.__version__)
+NUMPY_VERSION = Version(numpy.__version__)
 
 
 def pytest_addoption(parser):
@@ -122,6 +124,41 @@ def pytest_configure(config):
             ),
         )
 
+    if MPL_VERSION < Version("3.5.2"):
+        if MPL_VERSION < Version("3.3"):
+            try:
+                import PIL
+            except ImportError:
+                PILLOW_INSTALLED = False
+            else:
+                PILLOW_INSTALLED = True
+        else:
+            # pillow became a hard dependency in matplotlib 3.3
+            import PIL
+
+            PILLOW_INSTALLED = True
+        if PILLOW_INSTALLED and Version(PIL.__version__) >= Version("9.1"):
+            # see https://github.com/matplotlib/matplotlib/pull/22766
+            config.addinivalue_line(
+                "filterwarnings",
+                r"ignore:NONE is deprecated and will be removed in Pillow 10 \(2023-07-01\)\. "
+                r"Use Resampling\.NEAREST or Dither\.NONE instead\.:DeprecationWarning",
+            )
+            config.addinivalue_line(
+                "filterwarnings",
+                r"ignore:ADAPTIVE is deprecated and will be removed in Pillow 10 \(2023-07-01\)\. "
+                r"Use Palette\.ADAPTIVE instead\.:DeprecationWarning",
+            )
+
+    if NUMPY_VERSION < Version("1.19") and MPL_VERSION < Version("3.3"):
+        # This warning is triggered from matplotlib in exactly one test at the time of writing
+        # and exclusively on the minimal test env. Upgrading numpy or matplotlib resolves
+        # the issue, so we can afford to ignore it.
+        config.addinivalue_line(
+            "filterwarnings",
+            "ignore:invalid value encountered in less_equal:RuntimeWarning",
+        )
+
     if find_spec("astropy") is not None:
         # at the time of writing, astropy's wheels are behind numpy's latest
         # version but this doesn't cause actual problems in our test suite
