@@ -1,58 +1,79 @@
-"""
-RAMSES-specific IO
-
-
-
-"""
-
-#-----------------------------------------------------------------------------
-# Copyright (c) 2013, yt Development Team.
-#
-# Distributed under the terms of the Modified BSD License.
-#
-# The full license is in the file COPYING.txt, distributed with this software.
-#-----------------------------------------------------------------------------
-
+import os
 from collections import defaultdict
+from typing import Union
+
 import numpy as np
 
-from yt.utilities.io_handler import \
-    BaseIOHandler
+from yt._maintenance.deprecation import issue_deprecation_warning
+from yt.frontends.ramses.definitions import VAR_DESC_RE, VERSION_RE
+from yt.utilities.cython_fortran_utils import FortranFile
+from yt.utilities.exceptions import (
+    YTFieldTypeNotFound,
+    YTFileNotParseable,
+    YTParticleOutputFormatNotImplemented,
+)
+from yt.utilities.io_handler import BaseIOHandler
 from yt.utilities.logger import ytLogger as mylog
 from yt.utilities.physical_ratios import cm_per_km, cm_per_mpc
-from yt.utilities.cython_fortran_utils import FortranFile
-from yt.utilities.exceptions import YTFieldTypeNotFound, YTParticleOutputFormatNotImplemented, \
-    YTFileNotParseable
-import re
+
 
 def convert_ramses_ages(ds, conformal_ages):
+    issue_deprecation_warning(
+        msg=(
+            "The `convert_ramses_ages' function is deprecated. It should be replaced "
+            "by the `convert_ramses_conformal_time_to_physical_age' function."
+        ),
+        since="4.0.2",
+        removal="4.0.4",
+    )
+    return convert_ramses_conformal_time_to_physical_age(ds, conformal_ages)
+
+
+def convert_ramses_conformal_time_to_physical_age(
+    ds, conformal_time: np.ndarray
+) -> np.ndarray:
+    """
+    Convert conformal times (as defined in RAMSES) to physical age.
+
+    Arguments
+    ---------
+    ds : RAMSESDataset
+        The RAMSES dataset to use for the conversion
+    conformal_time : np.ndarray
+        The conformal time as read from disk
+
+    Returns
+    -------
+    physical_age : np.ndarray
+        The physical age in code units
+    """
     tf = ds.t_frw
     dtau = ds.dtau
     tauf = ds.tau_frw
-    tsim = ds.time_simu
     h100 = ds.hubble_constant
-    nOver2 = ds.n_frw/2
-    unit_t = ds.parameters['unit_t']
-    t_scale = 1./(h100 * 100 * cm_per_km / cm_per_mpc) / unit_t
+    nOver2 = ds.n_frw / 2
+    unit_t = ds.parameters["unit_t"]
+    tsim = ds.time_simu
+
+    t_scale = 1.0 / (h100 * 100 * cm_per_km / cm_per_mpc) / unit_t
 
     # calculate index into lookup table (n_frw elements in
     # lookup table)
-    dage =  1 + (10*conformal_ages/dtau)
-    dage = np.minimum(dage, nOver2 + (dage - nOver2)/10.)
+    dage = 1 + (10 * conformal_time / dtau)
+    dage = np.minimum(dage, nOver2 + (dage - nOver2) / 10.0)
     iage = np.array(dage, dtype=np.int32)
 
     # linearly interpolate physical times from tf and tauf lookup
     # tables.
-    t = ((tf[iage]*(conformal_ages - tauf[iage - 1]) /
-          (tauf[iage] - tauf[iage - 1])))
-    t = t + ((tf[iage-1]*(conformal_ages - tauf[iage]) /
-              (tauf[iage-1]-tauf[iage])))
-    return (tsim - t)*t_scale
+    t = tf[iage] * (conformal_time - tauf[iage - 1]) / (tauf[iage] - tauf[iage - 1])
+    t = t + (
+        tf[iage - 1] * (conformal_time - tauf[iage]) / (tauf[iage - 1] - tauf[iage])
+    )
+    return (tsim - t) * t_scale
 
 
-def _ramses_particle_file_handler(fname, foffsets, data_types,
-                                  subset, fields, count):
-    '''General file handler, called by _read_particle_subset
+def _ramses_particle_file_handler(fname, foffsets, data_types, subset, fields, count):
+    """General file handler, called by _read_particle_subset
 
     Parameters
     ----------
@@ -68,9 +89,10 @@ def _ramses_particle_file_handler(fname, foffsets, data_types,
         The fields to read
     count: integer
         The number of elements to count
-    '''
+    """
     tr = {}
     ds = subset.domain.ds
+    current_time = ds.current_time.in_units("code_time").v
     with FortranFile(fname) as fd:
         # We do *all* conversion into boxlen here.
         # This means that no other conversions need to be applied to convert
@@ -85,11 +107,14 @@ def _ramses_particle_file_handler(fname, foffsets, data_types,
             if field[1].startswith("particle_position"):
                 np.divide(tr[field], ds["boxlen"], tr[field])
             if ds.cosmological_simulation and field[1] == "particle_birth_time":
-                conformal_age = tr[field]
-                tr[field] = convert_ramses_ages(ds, conformal_age)
+                conformal_time = tr[field]
+                physical_age = convert_ramses_conformal_time_to_physical_age(
+                    ds, conformal_time
+                )
+                tr[field] = current_time - physical_age
                 # arbitrarily set particles with zero conformal_age to zero
                 # particle_age. This corresponds to DM particles.
-                tr[field][conformal_age == 0] = 0
+                tr[field][conformal_time == 0] = 0
     return tr
 
 
@@ -100,13 +125,12 @@ class IOHandlerRAMSES(BaseIOHandler):
         tr = defaultdict(list)
 
         # Set of field types
-        ftypes = set(f[0] for f in fields)
+        ftypes = {f[0] for f in fields}
         for chunk in chunks:
             # Gather fields by type to minimize i/o operations
             for ft in ftypes:
                 # Get all the fields of the same type
-                field_subs = list(
-                    filter(lambda f: f[0]==ft, fields))
+                field_subs = list(filter(lambda f: f[0] == ft, fields))
 
                 # Loop over subsets
                 for subset in chunk.objs:
@@ -127,8 +151,14 @@ class IOHandlerRAMSES(BaseIOHandler):
                         rv = subset.fill(fd, field_subs, selector, file_handler)
                     for ft, f in field_subs:
                         d = rv.pop(f)
-                        mylog.debug("Filling %s with %s (%0.3e %0.3e) (%s zones)",
-                            f, d.size, d.min(), d.max(), d.size)
+                        mylog.debug(
+                            "Filling %s with %s (%0.3e %0.3e) (%s zones)",
+                            f,
+                            d.size,
+                            d.min(),
+                            d.max(),
+                            d.size,
+                        )
                         tr[(ft, f)].append(d)
         d = {}
         for field in fields:
@@ -138,46 +168,49 @@ class IOHandlerRAMSES(BaseIOHandler):
 
     def _read_particle_coords(self, chunks, ptf):
         pn = "particle_position_%s"
-        fields = [(ptype, "particle_position_%s" % ax)
-                  for ptype, field_list in ptf.items()
-                  for ax in 'xyz']
+        fields = [
+            (ptype, f"particle_position_{ax}")
+            for ptype, field_list in ptf.items()
+            for ax in "xyz"
+        ]
         for chunk in chunks:
             for subset in chunk.objs:
                 rv = self._read_particle_subset(subset, fields)
                 for ptype in sorted(ptf):
-                    yield ptype, (rv[ptype, pn % 'x'],
-                                  rv[ptype, pn % 'y'],
-                                  rv[ptype, pn % 'z'])
+                    yield ptype, (
+                        rv[ptype, pn % "x"],
+                        rv[ptype, pn % "y"],
+                        rv[ptype, pn % "z"],
+                    )
 
     def _read_particle_fields(self, chunks, ptf, selector):
         pn = "particle_position_%s"
         chunks = list(chunks)
-        fields = [(ptype, fname) for ptype, field_list in ptf.items()
-                                 for fname in field_list]
+        fields = [
+            (ptype, fname) for ptype, field_list in ptf.items() for fname in field_list
+        ]
         for ptype, field_list in sorted(ptf.items()):
-            for ax in 'xyz':
+            for ax in "xyz":
                 if pn % ax not in field_list:
                     fields.append((ptype, pn % ax))
         for chunk in chunks:
             for subset in chunk.objs:
                 rv = self._read_particle_subset(subset, fields)
                 for ptype, field_list in sorted(ptf.items()):
-                    x, y, z = (np.asarray(rv[ptype, pn % ax], "=f8")
-                               for ax in 'xyz')
+                    x, y, z = (np.asarray(rv[ptype, pn % ax], "=f8") for ax in "xyz")
                     mask = selector.select_points(x, y, z, 0.0)
                     if mask is None:
-                       mask = []
+                        mask = []
                     for field in field_list:
                         data = np.asarray(rv.pop((ptype, field))[mask], "=f8")
                         yield (ptype, field), data
 
-
     def _read_particle_subset(self, subset, fields):
-        '''Read the particle files.'''
+        """Read the particle files."""
         tr = {}
 
         # Sequential read depending on particle type
-        for ptype in set(f[0] for f in fields):
+        for ptype in {f[0] for f in fields}:
 
             # Select relevant fiels
             subs_fields = filter(lambda f: f[0] == ptype, fields)
@@ -195,48 +228,49 @@ class IOHandlerRAMSES(BaseIOHandler):
                 raise YTFieldTypeNotFound(ptype)
 
             cosmo = self.ds.cosmological_simulation
-            if (ptype, 'particle_birth_time') in foffsets and cosmo:
-                foffsets[ptype, 'conformal_birth_time'] = \
-                    foffsets[ptype, 'particle_birth_time']
-                data_types[ptype, 'conformal_birth_time'] = \
-                    data_types[ptype, 'particle_birth_time']
+            if (ptype, "particle_birth_time") in foffsets and cosmo:
+                foffsets[ptype, "conformal_birth_time"] = foffsets[
+                    ptype, "particle_birth_time"
+                ]
+                data_types[ptype, "conformal_birth_time"] = data_types[
+                    ptype, "particle_birth_time"
+                ]
 
-            tr.update(_ramses_particle_file_handler(
-                fname, foffsets, data_types, subset, subs_fields,
-                count=count
-            ))
+            tr.update(
+                _ramses_particle_file_handler(
+                    fname, foffsets, data_types, subset, subs_fields, count=count
+                )
+            )
 
         return tr
 
 
-def _read_part_file_descriptor(fname):
+def _read_part_file_descriptor(fname: Union[str, "os.PathLike[str]"]):
     """
     Read a file descriptor and returns the array of the fields found.
     """
-    VERSION_RE = re.compile(r'# version: *(\d+)')
-    VAR_DESC_RE = re.compile(r'\s*(\d+),\s*(\w+),\s*(\w+)')
 
     # Mapping
-    mapping = [
-        ('position_x', 'particle_position_x'),
-        ('position_y', 'particle_position_y'),
-        ('position_z', 'particle_position_z'),
-        ('velocity_x', 'particle_velocity_x'),
-        ('velocity_y', 'particle_velocity_y'),
-        ('velocity_z', 'particle_velocity_z'),
-        ('mass', 'particle_mass'),
-        ('identity', 'particle_identity'),
-        ('levelp', 'particle_level'),
-        ('family', 'particle_family'),
-        ('tag', 'particle_tag')
+    mapping_list = [
+        ("position_x", "particle_position_x"),
+        ("position_y", "particle_position_y"),
+        ("position_z", "particle_position_z"),
+        ("velocity_x", "particle_velocity_x"),
+        ("velocity_y", "particle_velocity_y"),
+        ("velocity_z", "particle_velocity_z"),
+        ("mass", "particle_mass"),
+        ("identity", "particle_identity"),
+        ("levelp", "particle_level"),
+        ("family", "particle_family"),
+        ("tag", "particle_tag"),
     ]
-    # Convert in dictionary
-    mapping = {k: v for k, v in mapping}
+    # Convert to dictionary
+    mapping = {k: v for k, v in mapping_list}
 
-    with open(fname, 'r') as f:
+    with open(fname) as f:
         line = f.readline()
         tmp = VERSION_RE.match(line)
-        mylog.debug('Reading part file descriptor %s.' % fname)
+        mylog.debug("Reading part file descriptor %s.", fname)
         if not tmp:
             raise YTParticleOutputFormatNotImplemented()
 
@@ -249,7 +283,7 @@ def _read_part_file_descriptor(fname):
             for i, line in enumerate(f.readlines()):
                 tmp = VAR_DESC_RE.match(line)
                 if not tmp:
-                    raise YTFileNotParseable(fname, i+1)
+                    raise YTFileNotParseable(fname, i + 1)
 
                 # ivar = tmp.group(1)
                 varname = tmp.group(2)
@@ -258,7 +292,7 @@ def _read_part_file_descriptor(fname):
                 if varname in mapping:
                     varname = mapping[varname]
                 else:
-                    varname = 'particle_%s' % varname
+                    varname = f"particle_{varname}"
 
                 fields.append((varname, dtype))
         else:
@@ -266,29 +300,47 @@ def _read_part_file_descriptor(fname):
 
     return fields
 
-def _read_fluid_file_descriptor(fname):
+
+def _read_fluid_file_descriptor(fname: Union[str, "os.PathLike[str]"]):
     """
     Read a file descriptor and returns the array of the fields found.
     """
-    VERSION_RE = re.compile(r'# version: *(\d+)')
-    VAR_DESC_RE = re.compile(r'\s*(\d+),\s*(\w+),\s*(\w+)')
 
     # Mapping
-    mapping = [
-        ('density', 'Density'),
-        ('velocity_x', 'x-velocity'),
-        ('velocity_y', 'y-velocity'),
-        ('velocity_z', 'z-velocity'),
-        ('pressure', 'Pressure'),
-        ('metallicity', 'Metallicity'),
+    mapping_list = [
+        ("density", "Density"),
+        ("velocity_x", "x-velocity"),
+        ("velocity_y", "y-velocity"),
+        ("velocity_z", "z-velocity"),
+        ("pressure", "Pressure"),
+        ("metallicity", "Metallicity"),
+        # Add mapping for ionized species
+        # Note: we expect internally that these names use the HII, HeII,
+        #       HeIII, ... convention for historical reasons. So we need to map
+        #       the names read from `hydro_file_descriptor.txt` to this
+        #       convention.
+        # This will create fields like ("ramses", "HII") which are mapped
+        # to ("gas", "H_p1_fraction") in fields.py
+        ("H_p1_fraction", "HII"),
+        ("He_p1_fraction", "HeII"),
+        ("He_p2_fraction", "HeIII"),
     ]
-    # Convert in dictionary
-    mapping = {k: v for k, v in mapping}
 
-    with open(fname, 'r') as f:
+    # Add mapping for magnetic fields
+    mapping_list += [
+        (key, key)
+        for key in (
+            f"B_{dim}_{side}" for side in ["left", "right"] for dim in ["x", "y", "z"]
+        )
+    ]
+
+    # Convert to dictionary
+    mapping = {k: v for k, v in mapping_list}
+
+    with open(fname) as f:
         line = f.readline()
         tmp = VERSION_RE.match(line)
-        mylog.debug('Reading fluid file descriptor %s.' % fname)
+        mylog.debug("Reading fluid file descriptor %s.", fname)
         if not tmp:
             return []
 
@@ -301,7 +353,7 @@ def _read_fluid_file_descriptor(fname):
             for i, line in enumerate(f.readlines()):
                 tmp = VAR_DESC_RE.match(line)
                 if not tmp:
-                    raise YTFileNotParseable(fname, i+1)
+                    raise YTFileNotParseable(fname, i + 1)
 
                 # ivar = tmp.group(1)
                 varname = tmp.group(2)
@@ -310,11 +362,11 @@ def _read_fluid_file_descriptor(fname):
                 if varname in mapping:
                     varname = mapping[varname]
                 else:
-                    varname = 'hydro_%s' % varname
+                    varname = f"hydro_{varname}"
 
                 fields.append((varname, dtype))
         else:
-            mylog.error('Version %s', version)
+            mylog.error("Version %s", version)
             raise YTParticleOutputFormatNotImplemented()
 
     return fields
