@@ -1,3 +1,4 @@
+import sys
 import warnings
 from io import BytesIO
 
@@ -205,21 +206,36 @@ class ImagePlotMPL(PlotMPL):
             vmin=float(self.zmin) if self.zmin is not None else None,
             vmax=float(self.zmax) if self.zmax is not None else None,
         )
+
+        if MPL_VERSION < Version("3.2"):
+            # with MPL 3.1 we use np.inf as a mask instead of np.nan
+            # this is done in CoordinateHandler.sanitize_buffer_fill_values
+            # however masking with inf is problematic here when we search for the max value
+            # so here we revert to nan
+            # see https://github.com/yt-project/yt/pull/2517 and https://github.com/yt-project/yt/pull/3793
+            data[~np.isfinite(data)] = np.nan
+
         zmin = float(self.zmin) if self.zmin is not None else np.nanmin(data)
         zmax = float(self.zmax) if self.zmax is not None else np.nanmax(data)
 
         if cbnorm == "symlog":
             # if cblinthresh is not specified, try to come up with a reasonable default
-            min_abs_val = np.min(np.abs((zmin, zmax)))
-            if cblinthresh is None:
-                cblinthresh = np.nanmin(np.absolute(data)[data != 0])
-            elif zmin * zmax > 0 and cblinthresh < min_abs_val:
-                warnings.warn(
-                    f"Cannot set a symlog norm with linear threshold {cblinthresh} "
-                    f"lower than the minimal absolute data value {min_abs_val} . "
-                    "Switching to log norm."
-                )
-                cbnorm = "log10"
+            min_abs_val, max_abs_val = np.sort(
+                np.abs((np.nanmin(data), np.nanmax(data)))
+            )
+            if cblinthresh is not None:
+                if zmin * zmax > 0 and cblinthresh < min_abs_val:
+                    # see https://github.com/yt-project/yt/issues/3564
+                    warnings.warn(
+                        f"Cannot set a symlog norm with linear threshold {cblinthresh} "
+                        f"lower than the minimal absolute data value {min_abs_val} . "
+                        "Switching to log norm."
+                    )
+                    cbnorm = "log10"
+            elif min_abs_val > 0:
+                cblinthresh = min_abs_val
+            else:
+                cblinthresh = max_abs_val / 1000
 
         if cbnorm == "log10":
             cbnorm_cls = matplotlib.colors.LogNorm
@@ -251,33 +267,8 @@ class ImagePlotMPL(PlotMPL):
             transform = self.axes.transData
         else:
             transform = self._transform
-        if hasattr(self.axes, "set_extent"):
-            # CartoPy hangs if we do not set_extent before imshow if we are
-            # displaying a small subset of the globe.  What I believe happens is
-            # that the transform for the points on the outside results in
-            # infinities, and then the scipy.spatial cKDTree hangs trying to
-            # identify nearest points.
-            #
-            # Also, set_extent is defined by cartopy, so not all axes will have
-            # it as a method.
-            #
-            # A potential downside is that other images may change, but I believe
-            # the result of imshow is to set_extent *regardless*.  This just
-            # changes the order in which it happens.
-            #
-            # NOTE: This is currently commented out because it breaks in some
-            # instances.  It is left as a historical note because we will
-            # eventually need some form of it.
-            # self.axes.set_extent(extent)
 
-            # possibly related issue (operation order dependency)
-            # https://github.com/SciTools/cartopy/issues/1468
-
-            # in cartopy 0.19 (or 0.20), some intented behaviour changes produced an
-            # incompatibility here where default values for extent would lead to a crash.
-            # A solution is to let the transform object set the image extents internally
-            # see https://github.com/SciTools/cartopy/issues/1955
-            extent = None
+        self._validate_axes_extent(extent, transform)
 
         self.image = self.axes.imshow(
             data.to_ndarray(),
@@ -298,7 +289,7 @@ class ImagePlotMPL(PlotMPL):
                     10
                     ** np.arange(
                         np.rint(np.log10(cblinthresh)),
-                        np.ceil(np.log10(zmax)),
+                        np.ceil(np.log10(1.1 * zmax)),
                     )
                 )
             elif zmax <= 0.0:
@@ -307,19 +298,16 @@ class ImagePlotMPL(PlotMPL):
                 else:
                     offset = 1
 
-                yticks = (
-                    list(
-                        -(
-                            10
-                            ** np.arange(
-                                np.floor(np.log10(-zmin)),
-                                np.rint(np.log10(cblinthresh)) - offset,
-                                -1,
-                            )
+                yticks = list(
+                    -(
+                        10
+                        ** np.arange(
+                            np.floor(np.log10(-zmin)),
+                            np.rint(np.log10(cblinthresh)) - offset,
+                            -1,
                         )
                     )
-                    + [zmax]
-                )
+                ) + [zmax]
             else:
                 yticks = (
                     list(
@@ -337,15 +325,44 @@ class ImagePlotMPL(PlotMPL):
                         10
                         ** np.arange(
                             np.rint(np.log10(cblinthresh)),
-                            np.ceil(np.log10(zmax)),
+                            np.ceil(np.log10(1.1 * zmax)),
                         )
                     )
                 )
+            if yticks[-1] > zmax:
+                yticks.pop()
             self.cb.set_ticks(yticks)
         else:
             self.cb = self.figure.colorbar(self.image, self.cax)
         self.cax.tick_params(which="both", axis="y", direction="in")
 
+    def _validate_axes_extent(self, extent, transform):
+        # if the axes are cartopy GeoAxes, this checks that the axes extent
+        # is properly set.
+
+        if "cartopy" not in sys.modules:
+            # cartopy isn't already loaded, nothing to do here
+            return
+
+        from cartopy.mpl.geoaxes import GeoAxes
+
+        if isinstance(self.axes, GeoAxes):
+            # some projections have trouble when passing extents at or near the
+            # limits. So we only set_extent when the plot is a subset of the
+            # globe, within the tolerance of the transform.
+
+            # note that `set_extent` here is setting the extent of the axes.
+            # still need to pass the extent arg to imshow in order to
+            # ensure that it is properly scaled. also note that set_extent
+            # expects values in the coordinates of the transform: it will
+            # calculate the coordinates in the projection.
+            global_extent = transform.x_limits + transform.y_limits
+            thresh = transform.threshold
+            if all(
+                abs(extent[ie]) < (abs(global_extent[ie]) - thresh) for ie in range(4)
+            ):
+                self.axes.set_extent(extent, crs=transform)
+
     def _get_best_layout(self):
 
         # Ensure the figure size along the long axis is always equal to _figure_size
