@@ -381,7 +381,7 @@ class FITSImageData:
                         4.0
                         * np.pi
                         * self.mass_unit
-                        / (self.time_unit ** 2 * self.length_unit)
+                        / (self.time_unit**2 * self.length_unit)
                     )
                 else:
                     u = cgs_unit
@@ -399,21 +399,22 @@ class FITSImageData:
             else:
                 uq = None
 
-            if uq is not None and hasattr(self, "hubble_constant"):
-                # Don't store cosmology units
+            if uq is not None:
                 atoms = {str(a) for a in uq.units.expr.atoms()}
-                if str(uq.units).startswith("cm") or "h" in atoms or "a" in atoms:
+                if hasattr(self, "hubble_constant"):
+                    # Don't store cosmology units
+                    if str(uq.units).startswith("cm") or "h" in atoms or "a" in atoms:
+                        uq.convert_to_cgs()
+                if any(a.startswith("code") for a in atoms):
+                    # Don't store code units
+                    mylog.warning(
+                        "Cannot use code units of '%s' "
+                        "when creating a FITSImageData instance! "
+                        "Converting to a cgs equivalent.",
+                        uq.units,
+                    )
                     uq.convert_to_cgs()
 
-            if uq is not None and uq.units.is_code_unit:
-                mylog.warning(
-                    "Cannot use code units of '%s' "
-                    "when creating a FITSImageData instance! "
-                    "Converting to a cgs equivalent.",
-                    uq.units,
-                )
-                uq.convert_to_cgs()
-
             if attr == "length_unit" and uq.value != 1.0:
                 mylog.warning("Converting length units from %s to %s.", uq, uq.units)
                 uq = YTQuantity(1.0, uq.units)
